@@ -33,17 +33,7 @@
 #include <ofono/types.h>
 
 #include "atutil.h"
-
-void dump_response(const char *func, gboolean ok, GAtResult *result)
-{
-	GSList *l;
-
-	ofono_debug("%s got result: %d", func, ok);
-	ofono_debug("Final response: %s", result->final_or_pdu);
-
-	for (l = result->lines; l; l = l->next)
-		ofono_debug("Response line: %s", (char *) l->data);
-}
+#include "vendor.h"
 
 void decode_at_error(struct ofono_error *error, const char *final)
 {
@@ -76,6 +66,20 @@ gint at_util_call_compare_by_phone_number(gconstpointer a, gconstpointer b)
 				sizeof(struct ofono_phone_number));
 }
 
+gint at_util_call_compare_by_id(gconstpointer a, gconstpointer b)
+{
+	const struct ofono_call *call = a;
+	unsigned int id = GPOINTER_TO_UINT(b);
+
+	if (id < call->id)
+		return -1;
+
+	if (id > call->id)
+		return 1;
+
+	return 0;
+}
+
 gint at_util_call_compare(gconstpointer a, gconstpointer b)
 {
 	const struct ofono_call *ca = a;
@@ -149,7 +153,8 @@ GSList *at_util_parse_clcc(GAtResult *result)
 
 gboolean at_util_parse_reg_unsolicited(GAtResult *result, const char *prefix,
 					int *status,
-					int *lac, int *ci, int *tech)
+					int *lac, int *ci, int *tech,
+					unsigned int vendor)
 {
 	GAtResultIter iter;
 	int s;
@@ -164,15 +169,30 @@ gboolean at_util_parse_reg_unsolicited(GAtResult *result, const char *prefix,
 	if (g_at_result_iter_next_number(&iter, &s) == FALSE)
 		return FALSE;
 
-	if (g_at_result_iter_next_string(&iter, &str) == TRUE)
-		l = strtol(str, NULL, 16);
-	else
-		goto out;
+	switch (vendor) {
+	case OFONO_VENDOR_HUAWEI:
+		if (g_at_result_iter_next_unquoted_string(&iter, &str) == TRUE)
+			l = strtol(str, NULL, 16);
+		else
+			goto out;
 
-	if (g_at_result_iter_next_string(&iter, &str) == TRUE)
-		c = strtol(str, NULL, 16);
-	else
-		goto out;
+		if (g_at_result_iter_next_unquoted_string(&iter, &str) == TRUE)
+			c = strtol(str, NULL, 16);
+		else
+			goto out;
+
+		break;
+	default:
+		if (g_at_result_iter_next_string(&iter, &str) == TRUE)
+			l = strtol(str, NULL, 16);
+		else
+			goto out;
+
+		if (g_at_result_iter_next_string(&iter, &str) == TRUE)
+			c = strtol(str, NULL, 16);
+		else
+			goto out;
+	}
 
 	g_at_result_iter_next_number(&iter, &t);
 
@@ -194,7 +214,8 @@ out:
 
 gboolean at_util_parse_reg(GAtResult *result, const char *prefix,
 				int *mode, int *status,
-				int *lac, int *ci, int *tech)
+				int *lac, int *ci, int *tech,
+				unsigned int vendor)
 {
 	GAtResultIter iter;
 	int m, s;
@@ -204,21 +225,42 @@ gboolean at_util_parse_reg(GAtResult *result, const char *prefix,
 	g_at_result_iter_init(&iter, result);
 
 	while (g_at_result_iter_next(&iter, prefix)) {
+		gboolean r;
+
 		g_at_result_iter_next_number(&iter, &m);
 
 		/* Sometimes we get an unsolicited CREG/CGREG here, skip it */
 		if (g_at_result_iter_next_number(&iter, &s) == FALSE)
 			continue;
 
-		if (g_at_result_iter_next_string(&iter, &str) == TRUE)
-			l = strtol(str, NULL, 16);
-		else
-			goto out;
+		switch (vendor) {
+		case OFONO_VENDOR_HUAWEI:
+			r = g_at_result_iter_next_unquoted_string(&iter, &str);
 
-		if (g_at_result_iter_next_string(&iter, &str) == TRUE)
-			c = strtol(str, NULL, 16);
-		else
-			goto out;
+			if (r == TRUE)
+				l = strtol(str, NULL, 16);
+			else
+				goto out;
+
+			r = g_at_result_iter_next_unquoted_string(&iter, &str);
+
+			if (r == TRUE)
+				c = strtol(str, NULL, 16);
+			else
+				goto out;
+
+			break;
+		default:
+			if (g_at_result_iter_next_string(&iter, &str) == TRUE)
+				l = strtol(str, NULL, 16);
+			else
+				goto out;
+
+			if (g_at_result_iter_next_string(&iter, &str) == TRUE)
+				c = strtol(str, NULL, 16);
+			else
+				goto out;
+		}
 
 		g_at_result_iter_next_number(&iter, &t);
 
